/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright 2025 NXP
 */

#ifndef __DT_BINDINGS_CLOCK_IMX94_H
#define __DT_BINDINGS_CLOCK_IMX94_H

#define IMX94_CLK_DISPMIX_CLK_SEL	0

#define IMX94_CLK_DISPMIX_LVDS_CLK_GATE	0

#endif /* __DT_BINDINGS_CLOCK_IMX94_H */
