/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_ADI_AD7768_1_H
#define _DT_BINDINGS_ADI_AD7768_1_H

#define AD7768_TRIGGER_SOURCE_SYNC_OUT  0
#define AD7768_TRIGGER_SOURCE_GPIO3     1
#define AD7768_TRIGGER_SOURCE_DRDY      2

#endif /* _DT_BINDINGS_ADI_AD7768_1_H */
