/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2020 Pengutronix, Michael Tretter <kernel@pengutronix.de>
 */

#ifndef _DT_BINDINGS_CLOCK_XLNX_VCU_H
#define _DT_BINDINGS_CLOCK_XLNX_VCU_H

#define CLK_XVCU_ENC_CORE		0
#define CLK_XVCU_ENC_MCU		1
#define CLK_XVCU_DEC_CORE		2
#define CLK_XVCU_DEC_MCU		3
#define CLK_XVCU_NUM_CLOCKS		4

#endif /* _DT_BINDINGS_CLOCK_XLNX_VCU_H */
